package results;

import java.util.Collection;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

/** Collection of countries */
public final class Countries
{
  /** Hash table of the countries */
  private Hashtable<String,Country> countries =
    new Hashtable<String,Country> ();
  /** The locale used for the country names */
  private Locale locale;

  /** Constructor */
  public Countries ()
  {
    this (Locale.getDefault ());
  }

  /** Constructor
   * @param locale	Locale to use for fetching the country names
   */
  public Countries (Locale locale)
  {
    this.locale = locale;
    ResourceBundle labels = ResourceBundle.getBundle ("countries", locale);
    for (Enumeration<String> en = labels.getKeys (); en.hasMoreElements (); ) {
      String code = en.nextElement ();
      countries.put (code, new Country (code, labels.getString (code)));
    }
  }

  /** Change the locale
   * @param locale	new locale for country names
   */
  public void setLocale (Locale locale)
  {
    this.locale = locale;
    ResourceBundle labels = ResourceBundle.getBundle ("countries", locale);
    for (Enumeration<String> en = labels.getKeys (); en.hasMoreElements (); ) {
      String code = en.nextElement ();
      get (code).name = labels.getString (code);
    }
  }
  /** Determine the locale */
  public Locale getLocale () { return locale; }

  /** Get a country by code
   * @param code	Code of the country
   * @return		the Country object, or null
   */
  public Country get (String code)
  {
    return countries.get (code);
  }

  /** Get an enumeration of country codes */
  public Enumeration<String> keys ()
  {
    return countries.keys ();
  }

  /** Get the stored countries */
  public Collection<Country> values ()
  {
    return countries.values ();
  }
}
