package results;

import java.util.Comparator;

/** Sort mirrors according to their numbers */
final class MirrorSorter
  implements Comparator<Mirror>
{
  /** The error reporter object */
  private ErrorReporter reporter;

  /** Constructor
   * @param reporter	object for reporting non-unique numbers,
   *			or null if no reporting is expected
   */
  public MirrorSorter (ErrorReporter reporter)
  {
    this.reporter = reporter;
  }

  /** Compare two Mirror objects
   * @param m1		first mirror to compare
   * @param m2		second mirror to compare
   * @return		positive if the number of m1 is greater than m2,
   *			negative if the number of m1 is less than m2,
   *			or 0 if the mirror numbers are equal
   */
  public int compare (Mirror m1, Mirror m2)
  {
    final int diff = m1.number - m2.number;
    if (diff == 0 && m1 != m2 && reporter != null) {
      if (m1.number == 0)
	reporter.reportMissingNumber (m1);
      else
	reporter.reportRepeatedNumber (m1, m2);
    }
    return diff;
  }
}
