package results;

import java.util.Vector;

/** Team (collection of Competitors) */
public final class Team
{
  /** The series of this team */
  Series series;
  /** Name of the team */
  String name;
  /** List of competitors belonging to the team */
  Vector<Competitor> competitors;

  /** Constructor
   * @param series	The series of this team
   * @param name	Name of the team
   * @param competitors	Members of the team
   */
  Team (Series series, String name, Vector<Competitor> competitors) {
    this.series = series;
    this.name = name;
    this.competitors = competitors;
  }

  /** Constructor
   * @param series	The series of this team
   * @param name	Name of the team
   */
  public Team (Series series, String name) {
    this (series, name, new Vector<Competitor> ());
  }

  /** Register a competitor to the team
   * @param competitor	competitor to be added
   */
  public void add (Competitor competitor) {
    if (competitor.team != null)
      return;
    competitors.addElement (competitor);
    competitor.team = this;
    if (series != null) series.flagModified ();
  }

  /** Remove a competitor at the specified position */
  public void removeElementAt (int i) {
    Competitor competitor = competitors.elementAt (i);
    competitors.removeElementAt (i);
    competitor.team = null;
    if (series != null) series.flagModified ();
  }

  /** Replace a team member at the specified position
   * @param competitor	the replacement
   * @param i		the position
   */
  public void setElementAt (Competitor competitor, int i) {
    if (competitor.team != null)
      return;
    Competitor old = competitors.elementAt (i);
    if (competitor == old) return;
    old.team = null;
    competitor.team = this;
    competitors.setElementAt (competitor, i);
    if (series != null) series.flagModified ();
  }

  /** Remove a competitor from the team
   * @param competitor	competitor to be removed
   * @return		true if the competitor was removed
   */
  public boolean remove (Competitor competitor) {
    if (!competitors.remove (competitor))
      return false;
    competitor.team = null;
    if (series != null) series.flagModified ();
    return true;
  }

  /** Get the number of team members */
  public int size () {
    return competitors.size ();
  }

  /** Get the nth member of the team */
  public Competitor elementAt (int i) {
    return competitors.elementAt (i);
  }

  /** Get the name of the team */
  public String getName () { return name; }
  /** Set the name of the team */
  public void setName (String name) {
    if (this.name.equals (name)) return;
    this.name = name;
    if (series != null) series.flagModified ();
  }

  /** Convert the team to a printable string */
  public String toString () { return name; }

  /** Remove all competitors from the team */
  public void clear ()
  {
    for (Competitor competitor : competitors)
      competitor.team = null;
    competitors.clear ();
    if (series != null) series.flagModified ();
  }
}
