package results.ui;

import results.Series;
import results.Competitor;

import java.text.Collator;
import java.util.Comparator;
import java.util.TreeSet;
import javax.swing.JComboBox;

/** A combo box for picking a competitor */
public final class CompetitorBox
  extends JComboBox<Competitor>
{
  /** The serialization version */
  private static final long serialVersionUID = 1L;
  /** The empty competitor */
  private final static Competitor empty = new Competitor ("", null);

  /** Constructor
   * @param series	the series of the competitors
   */
  CompetitorBox (Series series) {
    updateCompetitors (series);
  }
  /** Rebuilds the competitor list.
   * @param series	the series of the competitors
   */
  void updateCompetitors (final Series series) {
    TreeSet<Competitor> set = new TreeSet<Competitor>
      (new Comparator<Competitor> ()
	{
	  final Collator collator = Collator.getInstance (series.getLocale ());
	  public int compare (Competitor c1, Competitor c2) {
	    return collator.compare (c1.getName (), c2.getName ());
	  }
	});
    for (Competitor competitor : series.getCompetitors ())
      set.add (competitor);
    removeAllItems ();
    addItem (empty);
    for (Competitor competitor : set)
      addItem (competitor);
  }

  /** Returns the current selected competitor.
   * @return	selected competitor, or null when the selection is empty
   */
  public Object getSelectedItem () {
    Object selected = super.getSelectedItem ();

    if (selected == empty)
      selected = null;

    return selected;
  }
}
