package results.ui;

import results.Contest;

import java.util.Locale;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

/** Table for viewing and editing contest data */
final class ContestTable
  extends JTable
{
  /** The serialization version */
  private static final long serialVersionUID = 1L;
  /** The series dialog containing this table */
  private SeriesDialog parent;
  /** The contest cell renderer */
  private DefaultTableCellRenderer rcontest;
  /** The format cell renderer */
  private DefaultTableCellRenderer rformat;

  /** The contest name column number */
  private static final int COLUMN_CONTEST = 0;
  /** The picture format column number */
  private static final int COLUMN_FORMAT = 1;

  /** Constructor
   * @param dialog	the containing series dialog
   */
  ContestTable (SeriesDialog dialog)
  {
    parent = dialog;
    setModel (new AbstractTableModel ()
      {
	/** The serialization version */
	private static final long serialVersionUID = 1L;
	public int getColumnCount () { return 2; }
	public int getRowCount () {
	  return parent == null ? 1 : parent.series.getNumContests () + 1;
	}
	public boolean isCellEditable (int row, int col) {
	  return true;
	}
	public Class<?> getColumnClass (int col) {
	  return String.class;
	}
	public Object getValueAt (int row, int col) {
	  final int rows = getRowCount ();
	  if (row < rows - 1) {
	    switch (col) {
	    case COLUMN_CONTEST:
	      return parent.series.getContest (row).getName ();
	    case COLUMN_FORMAT:
	      return parent.series.getContest (row).getFormat ();
	    }
	  }
	  return null;
	}
	public void setValueAt (Object value, int row, int col) {
	  Contest contest = null;
	  final int rows = getRowCount ();
	  if (row < rows - 1) {
	    contest = parent.series.getContest (row);
	    switch (col) {
	    case COLUMN_CONTEST:
	      if (value != null && !"".equals (value))
		contest.setName ((String) value);
	      else if (parent.confirm ("contest.remove", contest.getName ()))
		parent.remove (contest);
	      break;
	    case COLUMN_FORMAT:
	      contest.setFormat ((String) value);
	      if (value == null || "".equals (value) &&
		  "".equals (contest.getName ()))
		parent.remove (contest);
	      break;
	    }
	  }
	  else if (value != null && !"".equals (value)) {
	    switch (col) {
	    case COLUMN_CONTEST:
	      contest = new Contest ((String) value);
	      break;
	    case COLUMN_FORMAT:
	      (contest = new Contest ("")).setFormat ((String) value);
	      break;
	    }
	    if (contest != null)
	      parent.add (contest);
	  }
	}
	public String getColumnName (int col) {
	  switch (col) {
	  case COLUMN_CONTEST:
	    return parent.translator.lookup ("contest.nContest");
	  case COLUMN_FORMAT:
	    return parent.translator.lookup ("contest.nFormat");
	  }
	  return null;
	}
      });
    rcontest = new DefaultTableCellRenderer ();
    rformat = new DefaultTableCellRenderer ();
  }

  /** Get the selected contest
   * @return		the contest, or null if none has been selected
   */
  public Contest getSelected ()
  {
    if (parent == null) return null;
    int i = getSelectionModel ().getAnchorSelectionIndex ();
    return i >= 0 && i < parent.series.getNumContests ()
      ? parent.series.getContest (i)
      : null;
  }

  /** Set the locale for the table
   * @param locale	the new locale
   */
  public void setLocale (Locale locale)
  {
    super.setLocale (locale);
    if (parent == null)
      return;
    if (rcontest != null)
      rcontest.setToolTipText (parent.translator.lookup ("contest.tContest"));
    if (rformat != null)
      rformat.setToolTipText (parent.translator.lookup ("contest.tFormat"));
    tableChanged (new TableModelEvent
      (getModel (), TableModelEvent.HEADER_ROW));
  }

  /** Invoked when the table model changes */
  public void tableChanged (TableModelEvent e) {
    super.tableChanged (e);
    if (e == null || e.getFirstRow () == TableModelEvent.HEADER_ROW) {
      if (rcontest != null && rformat != null) {
	getColumnModel ().getColumn (COLUMN_CONTEST).setCellRenderer
	  (rcontest);
	getColumnModel ().getColumn (COLUMN_FORMAT).setCellRenderer
	  (rformat);
      }
    }
  }
}
