package results.ui;

import results.Countries;
import results.Country;

import java.text.Collator;
import java.util.TreeMap;
import javax.swing.JComboBox;

/** A combo box for picking a country code */
public final class CountryBox
  extends JComboBox<Country>
{
  /** The serialization version */
  private static final long serialVersionUID = 1L;
  /** The countries displayed in this box */
  private Countries countries;
  /** The default country for new data */
  private Country country;

  /** Constructor
   * @param countries	the countries this box displays
   */
  public CountryBox (Countries countries) {
    this.countries = countries;
    updateCountries ();
  }

  /** Rebuild the list */
  public void updateCountries () {
    TreeMap<String,Country> map = new TreeMap<String,Country>
      (Collator.getInstance (countries.getLocale ()));
    for (Country c : countries.values ())
      map.put (c.name, c);
    removeAllItems ();
    for (Country c : map.values ())
      addItem (c);
  }

  /** Get the selected country */
  public Object getSelectedItem ()
  {
    Country c = (Country) super.getSelectedItem ();
    if (c == null)
      return country == null ? (Country) getItemAt (0) : country;
    else
      return country = c;
  }

  /** Get the default country */
  public Country getDefault ()
  {
    return (Country) getSelectedItem ();
  }
}
