package results.ui;

import results.Country;
import results.Judge;

import java.util.Locale;
import javax.swing.DefaultCellEditor;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;

/** Table for viewing and editing judge data */
final class JudgeTable
  extends JTable
{
  /** The serialization version */
  private static final long serialVersionUID = 1L;
  /** The series dialog containing this table */
  private SeriesDialog parent;
  /** The country box */
  private CountryBox countrybox;
  /** The country cell renderer */
  private DefaultTableCellRenderer rcountry;
  /** The judge name cell renderer */
  private DefaultTableCellRenderer rjudge;

  /** The country column number */
  private static final int COLUMN_COUNTRY = 0;
  /** The judge name column number */
  private static final int COLUMN_JUDGE = 1;

  /** Constructor
   * @param dialog	the containing series dialog
   */
  JudgeTable (SeriesDialog dialog)
  {
    parent = dialog;
    setModel (new AbstractTableModel ()
      {
	/** The serialization version */
	private static final long serialVersionUID = 1L;
	public int getColumnCount () { return 2; }
	public int getRowCount () {
	  return parent == null ? 1 : parent.series.getNumJudges () + 1;
	}
	public boolean isCellEditable (int row, int col) {
	  return true;
	}
	public Class<?> getColumnClass (int col) {
	  switch (col) {
	  case COLUMN_COUNTRY:
	    return Country.class;
	  case COLUMN_JUDGE:
	    return String.class;
	  }
	  return null;
	}
	public Object getValueAt (int row, int col) {
	  final int rows = getRowCount ();
	  if (row < rows - 1) {
	    final Judge judge = parent.series.getJudge (row);
	    switch (col) {
	    case COLUMN_COUNTRY:
	      return judge.getCountry ();
	    case COLUMN_JUDGE:
	      return judge.getName ();
	    }
	  }
	  return null;
	}
	public void setValueAt (Object value, int row, int col) {
	  final int rows = getRowCount ();
	  Judge judge;
	  if (row < rows - 1)
	    judge = parent.series.getJudge (row);
	  else {
	    judge = new Judge ("", countrybox.getDefault ());
	    parent.series.add (judge);
	  }
	  switch (col) {
	  case COLUMN_COUNTRY:
	    if (value != null)
	      judge.setCountry ((Country) value);
	    break;
	  case COLUMN_JUDGE:
	    if (value == null || value.equals ("")) {
	      if (row >= rows - 1 ||
		  parent.confirm ("judge.remove", judge.getName ()))
		parent.series.remove (judge);
	    }
	    else
	      judge.setName ((String) value);
	    break;
	  }
	}
	public String getColumnName (int col) {
	  switch (col) {
	  case COLUMN_COUNTRY:
	    return parent.translator.lookup ("judge.nCountry");
	  case COLUMN_JUDGE:
	    return parent.translator.lookup ("judge.nJudge");
	  }
	  return null;
	}
      });
    countrybox = new CountryBox (parent.series.countries);
    setDefaultEditor (Country.class, new DefaultCellEditor (countrybox));
    setDefaultRenderer (Country.class,
			rcountry = new DefaultTableCellRenderer ());
    setDefaultRenderer (String.class,
			rjudge = new DefaultTableCellRenderer ());
  }

  public void setLocale (Locale locale)
  {
    super.setLocale (locale);
    if (countrybox != null)
      countrybox.updateCountries ();
    if (parent == null)
      return;
    if (rcountry != null)
      rcountry.setToolTipText (parent.translator.lookup ("judge.tCountry"));
    if (rjudge != null)
      rjudge.setToolTipText (parent.translator.lookup ("judge.tJudge"));
    tableChanged (null);
  }
}
