package results.ui;

import java.util.Locale;
import java.util.ResourceBundle;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;

/** A class that handles translations */
final class ResourceTranslator
{
  /** The resource bundle used for translations */
  private ResourceBundle bundle = null;

  /** Set the resource bundle
   * @param bundlename	name of the resource bundle
   * @param locale	the locale
   */
  protected final void setLocale (String bundlename, Locale locale) {
    bundle = ResourceBundle.getBundle (bundlename, locale);
  }

  /** Translate a button
   * @param button	the button to be translated
   * @param label	the translation label
   */
  protected final void translate (JButton button, String label) {
    String text = bundle.getString (label);
    int i = text.indexOf ('&');
    if (i >= 0) {
      button.setMnemonic (text.charAt (i + 1));
      text = text.substring (0, i) + text.substring (i + 1);
    }
    else
      button.setMnemonic (0);
    button.setText (text);
    button.setToolTipText (bundle.getString ("t" + label));
  }
  /** Translate a tab
   * @param tabs	the tabbed pane
   * @param index	index of the tab to translate
   * @param label	the translation label
   */
  protected final void translate (JTabbedPane tabs, int index, String label) {
    tabs.setTitleAt (index, bundle.getString (label));
    tabs.setToolTipTextAt (index, bundle.getString ("t" + label));
  }
  /** Translate a menu item
   * @param item	the menu
   * @param label	the translation label
   */
  protected static final void translate (JMenuItem item, String label) {
    int i = label.indexOf ('&');
    if (i >= 0) {
      item.setMnemonic (label.charAt (i + 1));
      label = label.substring (0, i) + label.substring (i + 1);
    }
    else
      item.setMnemonic (0);
    item.setText (label);
  }

  /** Look up a string
   * @param label	the translation label
   * @return		the translation
   */
  protected final String lookup (String label) {
    return bundle == null ? null : bundle.getString (label);
  }

  /** Display a confirmation (ok/cancel) dialog
   * @param parent	the parent dialog
   * @param label	the question
   * @param title	the dialog title
   * @return		true if the question was confirmed
   */
  protected boolean confirm (JFrame parent, String label, String title) {
    return
      JOptionPane.showOptionDialog (parent, bundle.getString (label),
				    title == null ? parent.getTitle () : title,
				    0, JOptionPane.QUESTION_MESSAGE,
				    null, new Object[] {
				      bundle.getString ("ok"),
				      bundle.getString ("cancel"),
				    },
				    bundle.getString ("cancel")) == 0;
  }

  /** Display a confirmation (yes/no/cancel) dialog
   * @param parent	the parent dialog
   * @param label	the question
   * @param title	the dialog title
   * @return		0=yes, 1=no, 2=cancel
   */
  protected int yesnocancel (JFrame parent, String label, String title) {
    return
      JOptionPane.showOptionDialog (parent, bundle.getString (label),
				    title == null ? parent.getTitle () : title,
				    0, JOptionPane.QUESTION_MESSAGE,
				    null, new Object[] {
				      bundle.getString ("yes"),
				      bundle.getString ("no"),
				      bundle.getString ("cancel"),
				    },
				    bundle.getString ("cancel"));
  }
}
