package results.ui;

import results.Series;
import results.Contest;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.Enumeration;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/** The main dialog for viewing and editing the data for a series */
final class SeriesDialog
  extends JFrame
{
  /** The serialization version */
  private static final long serialVersionUID = 1L;
  /** All created series dialogs */
  static Vector<SeriesDialog> dialogs = new Vector<SeriesDialog> ();
  /** The file chooser */
  private static JFileChooser filechooser;
  /** The series file description */
  private String description;
  /** The resource bundle for menu labels */
  private ResourceBundle menu;
  /** The error reporter dialog */
  private ErrorDialog reporter;
  /** The series file filter */
  private FileFilter seriesfilter = new FileFilter () {
      /** Accept files for display */
      public boolean accept (File f)
      {
	return
	  f.isDirectory () || f.getName ().endsWith (Series.FILE_EXTENSION);
      }
      /** Get the description of this filter */
      public String getDescription () { return description; }
    };

  /** The resource translator */
  ResourceTranslator translator = new ResourceTranslator ();
  /** The series */
  Series series;
  /** The panel tabs */
  private JTabbedPane tabs;
  /** The table dialogs */
  private JTable tables[];
  /** The "competitors" tab */
  private static final int TABS_COMPETITORS = 0;
  /** The "teams" tab */
  private static final int TABS_TEAMS = 1;
  /** The "judges" tab */
  private static final int TABS_JUDGES = 2;
  /** The "contests" tab */
  private static final int TABS_CONTESTS = 3;
  /** Number of tabs */
  private static final int TABS_COUNT = 4;

  /** The "file" menu */
  private static final int MENU_FILE = 0;
  /** The "file/new" menu */
  private static final int MENU_FILE_NEW = 0;
  /** The "file/open" menu */
  private static final int MENU_FILE_OPEN = 1;
  /** The "file/revert" menu */
  private static final int MENU_FILE_REVERT = 2;
  /** The "file/save" menu */
  private static final int MENU_FILE_SAVE = 3;
  /** The "file/save as" menu */
  private static final int MENU_FILE_SAVEAS = 4;
  /** The "file/language" menu */
  private static final int MENU_FILE_LANG = 6;
  /** The "file/national" checkbox */
  private static final int MENU_FILE_NATIONAL = 7;
  /** The "file/export html" menu */
  private static final int MENU_FILE_HTML = 8;
  /** The "file/close" menu */
  private static final int MENU_FILE_CLOSE = 10;
  /** The "file/exit" menu */
  private static final int MENU_FILE_EXIT = 11;
  
  /** Flag: enable the "look and feel" menu */
  private static final boolean enableLNF = false;
  /** The "look and feel" menu */
  private static final int MENU_LNF = 1;

  /** Constructor
   * @param series	the series whose dialog this is
   */
  SeriesDialog (Series series)
  {
    this.series = series;
    dialogs.add (this);
    setJMenuBar (createMenuBar ());
    setDefaultCloseOperation (DO_NOTHING_ON_CLOSE);

    filechooser = new JFileChooser (System.getProperty ("user.dir"));
    filechooser.resetChoosableFileFilters ();
    filechooser.addChoosableFileFilter (seriesfilter);
    filechooser.setFileSelectionMode (JFileChooser.FILES_ONLY);

    tabs = new JTabbedPane () {
	/** The serialization version */
	private static final long serialVersionUID = 1L;
	/** Invoked when the tab selection has been changed */
	protected void fireStateChanged () {
	  super.fireStateChanged ();
	  if (!SeriesDialog.this.isVisible ()) return;
	  stopEditing ();
	  switch (getSelectedIndex ()) {
	  case TABS_COMPETITORS:
	    ((CompetitorTable) tables[TABS_COMPETITORS]).updateTeams ();
	    break;
	  case TABS_TEAMS:
	    ((TeamTable) tables[TABS_TEAMS]).updateCompetitors ();
	    break;
	  case TABS_JUDGES:
	    break;
	  case TABS_CONTESTS:
	    ((MirrorTable) tables[TABS_COUNT]).tableChanged (null);
	    break;
	  }
	}
      };
    getContentPane ().add (tabs);

    tables = new JTable[TABS_COUNT + 1];
    tabs.addTab ("", null, createScroll
		 (tables[TABS_COMPETITORS] = new CompetitorTable (this)));
    tabs.addTab ("", null, createScroll
		 (tables[TABS_TEAMS] = new TeamTable (this)));
    tabs.addTab ("", null, createScroll
		 (tables[TABS_JUDGES] = new JudgeTable (this)));
    tables[TABS_CONTESTS] = new ContestTable (this);
    tables[TABS_COUNT] =
      new MirrorTable (this, (ContestTable) tables[TABS_CONTESTS]);
    tables[TABS_CONTESTS].getSelectionModel ().addListSelectionListener
      (new ListSelectionListener () {
	  public void valueChanged (ListSelectionEvent e) {
	    if (e.getValueIsAdjusting ()) return;
	    if (tables[TABS_COUNT].isEditing () &&
		tables[TABS_COUNT].getCellEditor ().stopCellEditing ())
	      tables[TABS_COUNT].editingStopped (null);
	    tables[TABS_COUNT].tableChanged (null);
	  }
	});
    JSplitPane split =
      new JSplitPane (JSplitPane.HORIZONTAL_SPLIT,
		      createScroll (tables[TABS_CONTESTS]),
		      createScroll (tables[TABS_COUNT]));
    tabs.addTab ("", null, split);
    setLocale (series.getLocale ());
    split.setDividerLocation (tables[TABS_CONTESTS].getPreferredSize ().width);
    pack ();
    setVisible (true);
  }

  /** Create a scroll pane.
   * @param table	the content
   * @return		the scroll pane
   */
  public static JScrollPane createScroll (JComponent component)
  {
    JScrollPane sp = new JScrollPane (component);
    /* Disable this for now in order to avoid excessively wide windows. */
    /*
    Dimension size = sp.getPreferredSize ();
    size.width = component.getPreferredSize ().width;
    sp.setPreferredSize (size);
    */
    return sp;
  }

  /** Create the menu bar
   * @return	the created menu bar
   */
  private JMenuBar createMenuBar () {
    JMenuBar mb = new JMenuBar ();
    MenuListener ml = new MenuListener ();
    // the "file" menu
    JMenu m = new JMenu ();
    // "file/new"
    m.add (new JMenuItem ()).addActionListener (ml);
    // "file/open"
    m.add (new JMenuItem ()).addActionListener (ml);
    // "file/revert"
    m.add (new JMenuItem ()).addActionListener (ml);
    // "file/save"
    m.add (new JMenuItem ()).addActionListener (ml);
    // "file/save as"
    m.add (new JMenuItem ()).addActionListener (ml);
    // separator
    m.add (new JSeparator ());
    // "file/language"
    m.add (createLocaleMenu (getLocale ()));
    // "file/national"
    m.add (new JCheckBoxMenuItem (null, series.isNational ())).
      addActionListener (ml);
    // "file/export html"
    m.add (new JMenuItem ()).addActionListener (ml);
    // separator
    m.add (new JSeparator ());
    // "file/close"
    m.add (new JMenuItem ()).addActionListener (ml);
    // "file/exit"
    m.add (new JMenuItem ()).addActionListener (ml);
    mb.add (m);
    if (enableLNF) {
      // the "look and feel" menu
      m = new JMenu ();
      UIManager.LookAndFeelInfo info[] = UIManager.getInstalledLookAndFeels ();
      ActionListener listener = new ActionListener ()
	{
	  public void actionPerformed (ActionEvent e) {
	    Component[] components =
	      SeriesDialog.this.getJMenuBar ().getMenu
	      (MENU_LNF).getMenuComponents ();
	    Object source = e.getSource ();
	    for (int i = 0; i < components.length; i++)
	      ((JRadioButtonMenuItem) components[i]).setSelected
		(components[i] == source);
	    try {
	      UIManager.setLookAndFeel (e.getActionCommand ());
	      SwingUtilities.updateComponentTreeUI (filechooser);
	      for (SeriesDialog d : SeriesDialog.dialogs)
		SwingUtilities.updateComponentTreeUI (d);
	    }
	    catch (Exception ex) {
	    }
	  }
	};
      LookAndFeel current = UIManager.getLookAndFeel ();
      String cname = current == null ? null : current.getClass ().getName ();
      if (cname == null) cname = "";
      for (int i = 0; i < info.length; i++) {
	try {
	  String lname = info[i].getClassName ();
	  LookAndFeel lnf = (LookAndFeel) Class.forName (lname).newInstance ();
	  if (lnf.isSupportedLookAndFeel ()) {
	    JMenuItem item =
	      current == null ? new JMenuItem (info[i].getName ())
	      : new JRadioButtonMenuItem (info[i].getName (),
					  lname.equals (cname));
	    m.add (item);
	    item.setActionCommand (info[i].getClassName ());
	    item.addActionListener (listener);
	  }
	}
	catch (Exception e) {}
      }
      mb.add (m);
    }
    return mb;
  }

  /** Create the locale menu
   * @param locale	the locale
   * @return		the menu
   */
  private JMenu createLocaleMenu (Locale locale) {
    JMenu lmenu = new JMenu ();
    ResourceBundle lang =
      ResourceBundle.getBundle ("lang", locale);
    ActionListener listener = new ActionListener ()
      {
	public void actionPerformed (ActionEvent e) {
	  setLocale (new Locale (e.getActionCommand (), "", ""));
	}
      };
    for (Enumeration<String> en = lang.getKeys (); en.hasMoreElements (); ) {
      String l = en.nextElement ();
      JRadioButtonMenuItem item =
	new JRadioButtonMenuItem (lang.getString (l),
				  l.equals (locale.getLanguage ()));
      item.setActionCommand (l);
      item.addActionListener (listener);
      lmenu.add (item);
    }
    return lmenu;
  }

  /** Change the language
   * @param locale	the new locale
   */
  public void setLocale (Locale locale) {
    super.setLocale (locale);
    if (series == null || translator == null)
      return;
    series.setLocale (locale);
    translator.setLocale ("main", locale);
    translator.translate (tabs, TABS_COMPETITORS, "aCompetitors");
    translator.translate (tabs, TABS_TEAMS, "aTeams");
    translator.translate (tabs, TABS_JUDGES, "aJudges");
    translator.translate (tabs, TABS_CONTESTS, "aContests");
    menu = ResourceBundle.getBundle ("menu", locale);
    JMenuBar mb = getJMenuBar ();
    JMenu m = mb.getMenu (MENU_FILE);
    translate (m, "file");
    translate (m.getItem (MENU_FILE_NEW), "file.new");
    translate (m.getItem (MENU_FILE_OPEN), "file.open");
    translate (m.getItem (MENU_FILE_REVERT), "file.revert");
    translate (m.getItem (MENU_FILE_SAVE), "file.save");
    translate (m.getItem (MENU_FILE_SAVEAS), "file.saveas");
    m.remove (MENU_FILE_LANG);
    translate ((JMenu) m.add (createLocaleMenu (locale), MENU_FILE_LANG),
	       "file.lang");
    translate (m.getItem (MENU_FILE_NATIONAL), "file.national");
    translate (m.getItem (MENU_FILE_HTML), "file.html");
    translate (m.getItem (MENU_FILE_CLOSE), "file.close");
    translate (m.getItem (MENU_FILE_EXIT), "file.exit");
    if (enableLNF) {
      m = mb.getMenu (MENU_LNF);
      translate (m, "lnf");
    }
    String title = series.getName ();
    if (title == null)
      title = translator.lookup ("nameless");
    setTitle (series.getTournament () + " - " + title);
    description = translator.lookup (".pts");
    filechooser.setLocale (locale);
    for (int i = tables.length; i-- > 0; )
      tables[i].setLocale (locale);
    synchronized (getTreeLock ()) {
      validateTree ();
      validate ();
    }
  }

  /** Process a window event */
  protected void processWindowEvent (WindowEvent e) {
    super.processWindowEvent (e);
    if (e.getID () == WindowEvent.WINDOW_CLOSING)
      close (true);
  }

  /** Close the dialog but do not exit */
  void closeNoExit ()
  {
    if (reporter != null) {
      reporter.setVisible (false);
      reporter.dispose ();
      reporter = null;
    }
    setVisible (false);
    dialogs.remove (this);
    dispose ();
  }

  /** Close the dialog
   * @param confirm	flag: confirm saving of modified data before closing
   */
  public void close (boolean confirm)
  {
    if (confirm && !confirmsave ())
      return;
    closeNoExit ();
    if (dialogs.isEmpty ())
      System.exit (0);
  }

  /** Translate a menu item
   * @param item	the menu item to translate
   * @param label	the translation
   */
  void translate (JMenuItem item, String label)
  {
    ResourceTranslator.translate (item, menu.getString (label));
  }

  /** Confirm before closing the dialog
   * @return	true if the dialog was closed
   */
  public boolean confirmsave ()
  {
    if (series.isModified ()) {
      switch (translator.yesnocancel (this, "close", null)) {
      case 0:
	if (!save (series.file))
	  return false;
	break;
      case 1:
	break;
      default:
	return false;
      }
    }
    return true;
  }

  /** Save the series to the specified file
   * @param file	the file (null == prompt user for file name)
   * @return		true if the file was saved
   */
  public boolean save (File file)
  {
    if (file == null) {
      if (filechooser.showSaveDialog (this) == JFileChooser.APPROVE_OPTION)
	file = filechooser.getSelectedFile ();
      else
	return false;
    }
    try {
      if (!file.exists () &&
	  file.getName ().lastIndexOf (Series.FILE_EXTENSION) == -1)
	file = new File (file.getName () + Series.FILE_EXTENSION);
      series.write (file);
      setTitle (series.getTournament () + " - " + series.getName ());
      return true;
    }
    catch (Exception ex) {
      JOptionPane.showMessageDialog (SeriesDialog.this, ex, file.getName (),
				     JOptionPane.ERROR_MESSAGE);
      return false;
    }
  }

  /** Display a confirmation (ok/cancel) dialog
   * @param label	the question
   * @param title	the dialog title
   * @return		true if the question was confirmed
   */
  protected boolean confirm (String label, String title)
  {
    return translator.confirm (this, label, title);
  }
  /** Display a confirmation (yes/no/cancel) dialog
   * @param label	the question
   * @param title	the dialog title
   * @return		0=yes, 1=no, 2=cancel
   */
  protected int yesnocancel (String label, String title)
  {
    return translator.yesnocancel (this, label, title);
  }

  /** Display a warning message
   * @param label	the warning text
   * @param title	title of the dialog box
   */
  protected void warn (String label, String title)
  {
    JOptionPane.showMessageDialog (this, translator.lookup (label),
				   title == null ? getTitle () : title,
				   JOptionPane.ERROR_MESSAGE);
  }

  /** Add a contest to the series
   * @param contest	contest to be added
   */
  public void add (Contest contest)
  {
    series.add (contest);
    ((MirrorTable) tables[TABS_COUNT]).tableChanged (null);
  }
  /** Remove a contest from the series
   * @param contest	contest to be removed
   */
  public void remove (Contest contest)
  {
    series.remove (contest);
    ((MirrorTable) tables[TABS_COUNT]).tableChanged (null);
  }

  /** Display a warning message
   * @param label	the warning text
   * @param title	title of the dialog box
   * @param args	arguments for the warning message
   */
  protected void warn (String label, String title, Object[] args)
  {
    MessageFormat mf = new MessageFormat ("");
    mf.setLocale (getLocale ());
    mf.applyPattern (translator.lookup (label));
    JOptionPane.showMessageDialog (this, mf.format (args),
				   title == null ? getTitle () : title,
				   JOptionPane.ERROR_MESSAGE);
  }

  /** Stop editing on all tables */
  private final void stopEditing ()
  {
    for (int i = tables.length; i-- > 0; )
      if (tables[i] != null && tables[i].isEditing () &&
	  tables[i].getCellEditor ().stopCellEditing ())
	tables[i].editingStopped (null);
  }

  /** Signal that all tables have changed */
  private final void tablesChanged ()
  {
    for (int i = tables.length; i-- > 0; )
      if (tables[i] != null)
	tables[i].tableChanged (null);
  }

  /** Generate HTML output
   * @return	true on success, false on failure
   */
  final boolean generateHTML () {
    boolean ok = false;
    if (reporter != null) {
      reporter.setVisible (false);
      reporter.dispose ();
    }
    try {
      series.generate (reporter = new ErrorDialog (SeriesDialog.this));
      ok = !reporter.reported;
    }
    catch (IOException ex) {
      JOptionPane.showMessageDialog
	(SeriesDialog.this, ex, series.toString (),
	 JOptionPane.ERROR_MESSAGE);
    }
    if (reporter.reported)
      reporter.setVisible (true);
    return ok;
  }

  /** Make the dialog to use the given series data.
   * @param series	the series to be reverted to
   */
  private void setSeries (Series series) {
    this.series = series;
    tablesChanged ();
    setLocale (series.getLocale ());
    ((JCheckBoxMenuItem) getJMenuBar ().getMenu (MENU_FILE).
     getItem (MENU_FILE_NATIONAL)).setState (series.isNational ());
  }

  private class MenuListener implements ActionListener
  {
    public void actionPerformed (ActionEvent e) {
      JMenuItem item = (JMenuItem) e.getSource ();
      int cmd = ((JPopupMenu) item.getParent ()).getComponentIndex (item);
      stopEditing ();
      switch (cmd) {
      case MENU_FILE_NEW:
	new SeriesDialog (new Series (series.getLocale ()));
	break;
      case MENU_FILE_REVERT:
	if (series.file != null) {
	  try {
	    Series s = new Series (series.getLocale ());
	    s.read (series.file);
	    setSeries (s);
	  }
	  catch (Exception ex) {
	    JOptionPane.showMessageDialog (SeriesDialog.this, ex,
					   series.file.getName (),
					   JOptionPane.ERROR_MESSAGE);
	  }

	  break;
	}
	// fall through if no file has been loaded
      case MENU_FILE_OPEN:
	if (filechooser.showOpenDialog (SeriesDialog.this) !=
	    JFileChooser.APPROVE_OPTION)
	  break;
	File f = filechooser.getSelectedFile ().getAbsoluteFile ();
	SeriesDialog sd = null;
	for (SeriesDialog d : SeriesDialog.dialogs) {
	  if (d.series.file.getAbsoluteFile ().equals (f)) {
	    if (!d.confirmsave ())
	      return;
	    sd = d;
	    break;
	  }
	}
	try {
	  Series s = new Series (getLocale ());
	  s.read (f);
	  if (sd != null) {
	    sd.setSeries (s);
	    sd.requestFocus ();
	  }
	  else
	    new SeriesDialog (s);
	}
	catch (Exception ex) {
	  JOptionPane.showMessageDialog
	    (SeriesDialog.this, ex,
	     filechooser.getSelectedFile ().getName (),
	     JOptionPane.ERROR_MESSAGE);
	}
	break;
      case MENU_FILE_SAVE:
	save (series.file);
	break;
      case MENU_FILE_SAVEAS:
	save (null);
	break;
      case MENU_FILE_NATIONAL:
	((JCheckBoxMenuItem) item).setState (series.toggleNational ());
	break;
      case MENU_FILE_HTML:
	if (series.file == null)
	  warn ("export.nameless", null);
	else
	  generateHTML ();
	break;
      case MENU_FILE_CLOSE:
	close (true);
	break;
      case MENU_FILE_EXIT:
	for (SeriesDialog dialog : dialogs)
	  if (!dialog.confirmsave ())
	    return;
	while (!dialogs.isEmpty ())
	  dialogs.elementAt (0).close (false);
	break;
      }
    }
  }
}
