package results.ui;

import results.Series;
import results.Team;

import java.text.Collator;
import java.util.Comparator;
import java.util.TreeSet;
import javax.swing.JComboBox;

/** A combo box for picking a team */
public final class TeamBox
  extends JComboBox<Team>
{
  /** The serialization version */
  private static final long serialVersionUID = 1L;
  /** The empty team */
  private final static Team empty = new Team (null, "");

  TeamBox (Series series) {
    addTeams (series);
  }

  /** Rebuilds the team list.
   * @param series	the series of the teams
   */
  void updateTeams (Series series) {
    removeAllItems ();
    addTeams (series);
  }

  /** Populates the team list.
   * @param series	the series of the teams
   */
  void addTeams (final Series series) {
    TreeSet<Team> set = new TreeSet<Team>
      (new Comparator<Team> ()
	{
	  final Collator collator = Collator.getInstance (series.getLocale ());
	  public int compare (Team t1, Team t2) {
	    return collator.compare (t1.getName (), t2.getName ());
	  }
	});
    for (Team team : series.getTeams ())
      set.add (team);
    addItem (empty);
    for (Team team : set)
      addItem (team);
  }

  /** Returns the current selected team.
   * @return	selected team, or null when the selection is empty
   */
  public Object getSelectedItem () {
    Object selected = super.getSelectedItem ();

    if (selected == empty)
      selected = null;

    return selected;
  }
}
